import { useUserStore, useAppStore } from '@/store';
import axios, {
  AxiosRequestConfig,
  AxiosResponse,
  AxiosResponseHeaders,
  Method,
} from 'axios';
import qs from 'qs';
import { saveAs } from 'file-saver';
import { Message, Modal } from '@arco-design/web-vue';
import { getToken } from '@/utils/auth';
import router from '@/router';
import { LOCALE_ENUM } from '@/locale';
import { MainAxiosInstance, ResponseData } from './index.d';
import { RESPONSE_CODE } from './enum';

// 防止多个登录失效弹窗
let modalInstance: any;
let tokenExpired = false;

function download(blob: Blob, name: string) {
  saveAs(blob, name);
}
// create an axios instance
const service: MainAxiosInstance = axios.create({
  baseURL: import.meta.env.VITE_BASE_API as string, // url = base url + request url
  // withCredentials: true, // send cookies when cross-domain requests
  timeout: 1000 * 30,
  headers: { 'Content-Type': 'application/json' },
});
const { CancelToken } = axios;
const tokenSet = new Set();
// request interceptor
service.interceptors.request.use(
  (config: any) => {
    // token
    if (config.method === 'get') {
      config.paramsSerializer = (params: Record<string, any>) => {
        return qs.stringify(params, { arrayFormat: 'repeat' });
      };
    } else {
      const appStore = useAppStore();
      appStore.setloadding(true);
    }
    const token = getToken();
    if (token) {
      (config.headers as any)['X-Token'] = token;
    }
    // 国际化
    (config.headers as any).lang =
      localStorage.getItem('arco-locale') || LOCALE_ENUM.CN;
    config.cancelToken = new CancelToken(function executor(c) {
      // executor 函数接收一个 cancel 函数作为参数
      config.cancelFn = c;
      if (tokenExpired) {
        c();
      } else {
        tokenSet.add(c);
      }
    });
    return config;
  },
  (error) => {
    // do something with request error
    return Promise.reject(error);
  }
);
interface AxiosResponseAdd extends AxiosResponse {
  code?: number;
}
// response interceptor
service.interceptors.response.use(
  (response: AxiosResponse) => {
    if (tokenExpired) {
      tokenSet.forEach((fn: any) => fn());
      tokenSet.clear();
    } else {
      tokenSet.delete((response.config as any).cancelFn);
    }
    const res: ResponseData<Record<string, any> | string> | Blob =
      response.data;
    const appStore = useAppStore();
    appStore.setloadding(false);
    if (res instanceof Blob) {
      // 匹配文件名
      const fileNameArr = (
        (response.headers as AxiosResponseHeaders)['content-disposition']
          ? response.headers['content-disposition'].match(/filename=(\S*)/)
          : [null, Date.now()]
      ) as string[];

      download(res as unknown as Blob, decodeURI(fileNameArr[1]));
      return res;
    }
    // oauth接口需要特殊处理，响应成功没有code
    if (!('code' in res)) {
      return res;
    }
    // 根据业务完善
    if (res.code !== RESPONSE_CODE.SUCCESS) {
      // 登录失效
      if (res.code === RESPONSE_CODE.EXPIRED) {
        tokenExpired = true;
        const userStore = useUserStore();
        userStore.stopRefreshToken();
        if (!modalInstance) {
          modalInstance = Modal.confirm({
            title: '系统提示',
            content: res.message,
            onClose() {
              tokenExpired = false;
              userStore.frontLogout();
              router.push('/login');
              modalInstance = undefined;
            },
          });
        }
      }
      if (res.message) {
        // 提示错误信息
        Message.error(res.message);
      }
      return Promise.reject(res);
    }
    return res;
  },
  (error) => {
    const appStore = useAppStore();
    appStore.setloadding(false);
    if (error.message) {
      // 提示错误信息
      Message.error(error.message);
    }
    return Promise.reject(error);
  }
);

const exportFile = (url: string, data: any, methods = 'get') => {
  const config: AxiosRequestConfig<any> = {
    method: methods as Method,
    url, // 请求地址
    responseType: 'blob', // 表明返回服务器返回的数据类型,
    data: {},
    params: {},
  };
  if (methods === 'post') {
    config.data = data;
  } else {
    config.params = data;
  }
  return service.request(config);
};
/**
 * 方法扩展：导出excel
 * @param url
 * @param data
 * @param method
 * @param filename
 */
service.exportFile = async (url: string, data: any, method: Method) => {
  method = (method || 'get').toLowerCase() as Method;
  if (method === 'post') {
    await exportFile(url, data, 'post');
  } else {
    await exportFile(url, data);
  }
};

export default service;
