import { ref, computed, Ref } from 'vue';
import { Modal, Message } from '@arco-design/web-vue';
import deepmerge from 'deepmerge';
import { useIntervalFn } from '@vueuse/core';
import { useAppStore } from '@/store';
import { OPERATE_TYPE } from '@/constant/user';
import { noop, parseRangeFields, resetObj } from './helper';
import useAddOrUpdate from './addOrUpdate';

interface UseTableOption {
  // api相关，api优先级 pageApi > apiObj.page
  // todo ts如何定义class(typeof BaseApi)的实例对象？
  apiObj?: any; // api对象, 继承自BaseApi的实例对象
  // 分页
  pageApi?: (data: any) => any;
  // 详情
  getApi?: (data: any) => any;
  // 新增
  addApi?: (data: any) => any;
  // 列表
  listApi?: (data: any) => any;
  // 编辑
  updateApi?: (data: any) => any;
  // 删除
  delApi?: (data: any) => any;
  // 查询条件相关
  // 查询表单项配置
  searchConfig?: Array<Record<string, any>>;
  // 查询表单对象
  searchForm?: Record<string, any>;
  // 查询表单配置
  searchFormSetting?: Record<string, any>;
  // 扩展方法: 格式化查询参数
  formatSearchParams?: (data: any) => Record<string, any>;
  // 表格相关
  // 进入页面是否立即请求数据
  immediateGetTableData?: boolean;
  // 是否以行列表为详情进行回显
  useRowAsDetails?: boolean;
  // 表格数据
  tableData: Ref<any[]>;
  // 表格列配置
  tableColumns?: Array<any>;
  // 表格配置
  tableSetting?: Record<string, any>;
  // 弹窗相关
  // 弹窗对象
  dialogForm?: Ref<Record<string, any>>;
  // 弹窗表单项配置
  dialogFormConfig?: Array<any>;
  // 弹窗表单校验规则
  dialogFormRules?: Record<string, any>;
  // 弹窗配置
  dialogSetting?: Record<string, any>;
  // 扩展方法: 格式化详情
  formatDetailsData?: (data: any) => Record<string, any>;
  // 扩展方法: 格式化提交参数
  formatSubmitData?: (data: any) => Record<string, any>;
  // 点击新增按钮之前调用的方法，可用于赋默认值
  onAddCallback?: () => void;
  // 删除异常
  onDelError?: (error: any) => void;
  // 展示弹窗后的回调
  onShowCallback?: () => void;
  // 获取表格数据之后
  onFormateTableData?: (data: any) => void;
  // 获取表格数据之后
  onGetTableDataCallback?: (data: any) => void;
  // 是否分页
  pagination?: boolean;
  // 列表是否轮询
  polling?: boolean;
  // 轮询间隔
  pollingInterval?: number;
  [key: string]: any;
}
const appStore = useAppStore();
export const dialogFormDefaultSetting = {
  'mask-closable': false,
  'footer': false,
  'unmountOnClose': true,
};

export default function useTable(options: UseTableOption) {
  // 查询条件表单属性
  const mergedSearchFormSetting = ref({});
  // 表格属性
  const mergedTableSetting = ref<Record<string, any>>({});
  // 弹窗属性
  const mergedDialogSetting = ref({});
  // 是否显示编辑弹窗
  const dialogVisible = ref(false);
  // 当前操作类型，根据类型显示不同的弹窗标题
  // const curOperate = ref<OPERATE_TYPE>(OPERATE_TYPE.ADD);
  const {
    searchForm,
    searchFormSetting = {},
    tableSetting = {},
    dialogFormSetting = {},
    dialogForm,
    apiObj,
    tableData,
    pageApi,
    getApi,
    addApi,
    listApi,
    updateApi,
    delApi,
    polling = false,
    pollingInterval = 30 * 1000,
    pagination = true,
    immediateGetTableData = true,
    useRowAsDetails = false,
    onAddCallback = noop,
    onShowCallback = noop,
    onDelError = noop,
    onGetTableDataCallback = noop,
    onFormateTableData = noop,
    formatDetailsData = noop,
    formatSearchParams = noop,
    formatSubmitData = noop,
    getDetailKey = 'id',
  } = options as any;

  const { onSubmit: onAddOrUpdateSubmit, initModel } = useAddOrUpdate({
    getDetailKey,
    apiObj,
    getApi,
    addApi,
    updateApi,
    model: dialogForm,
    formatDetailsData,
    formatSubmitData,
  });

  const searchFormRef = ref();

  function getSearchParams() {
    const { pagination: p } = mergedTableSetting.value as any;
    const params = parseRangeFields(searchForm);
    if (pagination) {
      params.current = p.current;
      params.size = p['page-size'];
    }
    return formatSearchParams(params);
  }

  function setTableLoading(loading = false) {
    // 轮询查询不改变loading状态，防止页面闪烁
    if (polling) return;
    (mergedTableSetting.value as any).loading = loading;
  }

  // 防止重复请求
  const isTableLoading = computed(
    () => (mergedTableSetting.value as any).loading === true
  );

  async function getTableData() {
    if (isTableLoading.value) return;
    try {
      setTableLoading(true);
      const params = getSearchParams();
      let res;
      if (pagination) {
        // 分页调用page接口
        res = await (pageApi ? pageApi(params) : apiObj.page(params));
        tableData.value = onFormateTableData(res.data.records);
        (mergedTableSetting.value as any).pagination.total = +res.data.total;
      } else {
        // 不分页调用list接口
        res = await (listApi ? listApi(params) : apiObj.list(params));
        tableData.value = onFormateTableData(res.data);
      }
      onGetTableDataCallback(res.data, mergedTableSetting);
      // console.info('返回数据:', res);
    } finally {
      setTableLoading(false);
    }
  }

  function onSearch() {
    // 查询将页码重置为1
    if (pagination) (mergedTableSetting.value as any).pagination.current = 1;
    getTableData();
  }

  function onAdd() {
    onAddCallback(dialogForm);
    // todo 暂时这样处理，新增如果有排序，默认设置999
    if ('sort' in dialogForm.value) {
      dialogForm.value.sort = 999;
    }
    // curOperate.value = OPERATE_TYPE.ADD;
    appStore.updateCurOperate(OPERATE_TYPE.ADD);
    dialogVisible.value = true;
  }

  function onCancel() {
    dialogVisible.value = false;
  }

  async function onShow(obj: Record<string, any>) {
    // 支持详情根据行列表回显
    let data;
    if (useRowAsDetails) {
      data = { ...obj };
      dialogForm.value = formatDetailsData(data);
    } else {
      await initModel(obj[getDetailKey]);
    }
    onShowCallback(dialogForm);
    dialogVisible.value = true;
    // curOperate.value = OPERATE_TYPE.SHOW;
    appStore.updateCurOperate(OPERATE_TYPE.SHOW);
  }

  async function onEdit(obj: Record<string, any>, cb?: any) {
    await onShow(obj);
    // curOperate.value = OPERATE_TYPE.EDIT;
    appStore.updateCurOperate(OPERATE_TYPE.EDIT);
    if (cb) cb();
  }

  async function onSubmit() {
    onAddOrUpdateSubmit(() => {
      // 操作成功，刷新列表
      onSearch();
      // 关闭弹窗
      dialogVisible.value = false;
    });
  }

  function onDel(obj: Record<string, any>) {
    Modal.confirm({
      title: '系统提示',
      content: '请确认删除,删除后不可恢复!',
      closable: true,
      async onOk() {
        try {
          const res = delApi
            ? await delApi(obj[getDetailKey])
            : await apiObj.del(obj[getDetailKey]);
          Message.success(res.message);
          onSearch();
        } catch (error) {
          onDelError(error);
        }
      },
    });
  }

  // 重置dialogForm, 一般用于弹窗关闭,清空表单
  const resetDialogForm = () => {
    resetObj(dialogForm.value);
  };

  const searchFormDefaultSetting = {
    'layout': 'inline',
    'confirm-btn-text': '查询',
    'show-cancel': false,
  };

  const tableDefaultSetting: Record<string, any> = {
    'stripe': true,
    'loading': false,
    'row-key': 'id',
    'hide-expand-button-on-empty': true,
    'pagination': {
      'total': 0,
      'current': 1,
      'page-size': 10,
      'page-size-options': [10, 20, 30, 40, 50],
      'show-total': true,
      'show-more': true,
      'show-page-size': true,
    },
    'scroll': { x: '100%', y: '100%' },
    'onExpand': (rowKey: string) => {
      const { expandedRowKeys } = mergedTableSetting.value.expandable;
      if (expandedRowKeys.includes(rowKey)) {
        expandedRowKeys.splice(expandedRowKeys.indexOf(rowKey), 1);
      } else {
        expandedRowKeys.push(rowKey);
      }
    },
    'onPageSizeChange': (pageSize: number) => {
      (mergedTableSetting.value as any).pagination['page-size'] = pageSize;
      onSearch();
    },
    'onPageChange': (page: number) => {
      (mergedTableSetting.value as any).pagination.current = page;
      getTableData();
    },
    'page-position': 'br',
  };

  // 查询条件表单属性
  mergedSearchFormSetting.value = deepmerge(
    searchFormDefaultSetting,
    searchFormSetting
  );

  // 表格属性
  mergedTableSetting.value = deepmerge(tableDefaultSetting, tableSetting);
  if (!pagination) (mergedTableSetting.value as any).pagination = false;

  // 弹窗属性
  mergedDialogSetting.value = deepmerge(
    dialogFormDefaultSetting,
    { onClose: resetDialogForm },
    dialogFormSetting
  );

  const dialogTitle = computed(() => {
    switch (appStore.curOperate) {
      case OPERATE_TYPE.ADD:
        return '新增';
      case OPERATE_TYPE.EDIT:
        return '编辑';
      case OPERATE_TYPE.SHOW:
        return '详情';
      case OPERATE_TYPE.ROLE_CONFIG_AUTHOR:
        return '配置权限';
      default:
        return '-';
    }
  });

  const validateAndGetTableData = () => {
    if (searchFormRef.value) {
      searchFormRef.value.submit();
    } else {
      getTableData();
    }
  };

  if (immediateGetTableData) {
    onMounted(validateAndGetTableData);
  }

  if (polling) {
    useIntervalFn(validateAndGetTableData, pollingInterval);
  }
  return {
    searchFormRef,
    resetDialogForm,
    dialogTitle,
    searchForm,
    onSearch,
    onAdd,
    onCancel,
    onShow,
    onEdit,
    onDel,
    onSubmit,
    getTableData,
    dialogVisible,
    searchFormSetting: mergedSearchFormSetting,
    tableSetting: mergedTableSetting,
    dialogSetting: mergedDialogSetting,
  };
}
