/* eslint-disable no-console */
import router from '@/router';
import { defineStore } from 'pinia';
import { setToken, clearToken, menuTree2Routes } from '@/utils/auth';
import { login, refreshToken } from '@/api/module/LoginApi';
import AuthApi from '@/api/module/AuthApi';
import AccountApi from '@/views/background/account/api/AccountApi';
import { ROLE_TYPE, SYSTEM_ADMIN } from '@/constant/role';

// eslint-disable-next-line no-underscore-dangle
const _window: any = window;

const useUserStore = defineStore('user', {
  state: (): any => ({
    // token相关信息
    authInfo: undefined,
    // 用户基本信息
    userInfo: undefined,
    tokenExpiredTime: 0,
    refreshTokenTimerId: undefined,
    roleTypes: [],
    menu: [],
    sidebarMenu: [],
    routes: [],
    permissions: [],
  }),

  getters: {
    isSuperAdmin(): boolean {
      let flag = false;
      try {
        flag =
          this.roleTypes?.includes(ROLE_TYPE.FREIGHT_STATION) ||
          this.roleTypes?.includes(SYSTEM_ADMIN);
      } catch (error) {
        flag = false;
      }
      return flag;
    },
  },

  actions: {
    // Set user's information
    setInfo(data: Record<string, any>) {
      this.userInfo = data?.user;
      this.roleTypes = data?.roleTypes;
      this.permissions = data?.permissions;
    },

    // Reset user's information
    resetInfo() {
      this.$reset();
    },

    setMenu(menu: []) {
      this.menu = menu;
    },

    setSidebarMenu(menu: []) {
      this.sidebarMenu = menu;
    },

    setRoutes(routes: []) {
      this.routes = routes;
    },

    // 初始化用户菜单
    initMenu(menu: []) {
      // const res = await MenuApi.tree()
      const { routes, sidebarMenu } = menuTree2Routes(menu);

      routes.forEach((route: any) => {
        router.addRoute(route);
      });
      this.setMenu(menu);
      this.setSidebarMenu(sidebarMenu);
      this.setRoutes(routes);
      return menu;
    },
    /**
     * @description: 获取用户信息
     * @param {boolean} isLogin 是否由登录页面触发
     * @return:
     */
    async info(isLogin?: boolean) {
      const res = await AccountApi.info();
      this.setInfo(res.data);
      const config: any = JSON.parse(localStorage.getItem('config') || '{}');
      if (!config?.screenEnabled) return res.data;
      // if (isLogin) {
      //   this.initTrace(res.data?.user);
      //   this.startTrace();
      // } else {
      //   // 刷新静态资源可能未执行完 故延迟调用方法
      //   setTimeout(() => {
      //     _window.trace = _window.webtrace();
      //     console.log('>>>>>>实例化', _window.trace?.getSessionId());
      //     this.startTrace();
      //   }, 200);
      // }
      return res.data;
    },
    initTrace(data: any) {
      try {
        _window.trace = _window.webtrace();
        console.log('>>>>>>实例化', _window.trace?.getSessionId());
        _window.trace.init({
          channel: 'SZJC',
          param: {
            productName: 'SZJC',
            username: data?.userCode ?? '',
            mobile: data?.mobile ?? '',
          },
        });
        console.log('>>>>>>初始化', _window.trace?.getSessionId());
      } catch (error) {
        console.log('录制初始化失败：', error);
      }
    },
    startTrace() {
      _window.trace?.start();
      console.log('>>>>>>开始录制', _window.trace?.getSessionId());
    },
    stopTrace() {
      _window.trace?.stop(`SZJC-${Date.now()}`);
      console.log('>>>>>>结束录制', _window.trace?.getSessionId());
    },
    // Get user's routers
    async getRouters() {
      const res = await AccountApi.getRouters();
      this.initMenu(res.data);
      return res.data;
    },
    // 处理登录返回信息
    handleAuthinfo(data: any) {
      // 设置过期时间戳
      this.setTokenExpiredTime(Date.now() + data.expires_in * 1000);
      this.authInfo = data;
      this.permissions = data.permissions;
      // 设置token需要拼接
      setToken(`${data.tokenHead} ${data.token}`);
    },

    // Login
    async login(loginForm: any) {
      try {
        const { data, code }: any = await login(loginForm);
        if (code === 200) {
          this.handleAuthinfo(data);
        }
      } catch (err) {
        clearToken();
        throw err;
      }
    },

    // 前端登出
    frontLogout() {
      this.stopRefreshToken();
      this.resetInfo();
      clearToken();
      // this.stopTrace();
    },
    // Logout
    async logout() {
      try {
        // 调用后台注销接口
        await AuthApi.logout();
      } finally {
        this.frontLogout();
      }
    },

    setTokenExpiredTime(timestamp: number) {
      this.tokenExpiredTime = timestamp;
    },

    // 开始refresh token定时器
    async startRefreshToken() {
      // 轮询检查token有效期
      clearInterval(this.refreshTokenTimerId);
      // 一分钟轮询一次，当有效期小于5分钟时，刷新token
      this.refreshTokenTimerId = setInterval(() => {
        if (this.tokenExpiredTime - Date.now() < 1000 * 60 * 5) {
          refreshToken({
            grant_type: 'refresh_token',
            refresh_token: this.authInfo.refresh_token,
            client_id: 'mms',
          })
            .then((res) => {
              this.handleAuthinfo(res);
            })
            .catch((err) => {
              // console.error('refresh token fail:', err);
              return Promise.reject(err);
            });
        }
      }, 1000 * 60);
    },

    stopRefreshToken() {
      clearInterval(this.refreshTokenTimerId);
      this.refreshTokenTimerId = undefined;
    },
  },

  persist: true,
});

export default useUserStore;
