import axios, { AxiosInstance } from 'axios';
import md5 from './md5-file';

const { CancelToken } = axios;
const request = axios.create({
  timeout: 0,
});
// 添加请求拦截器
request.interceptors.request.use(
  (config) => {
    // 在发送请求之前做些什么
    return config;
  },
  (error) => {
    // 对请求错误做些什么
    return Promise.reject(error);
  }
);

// 添加响应拦截器
request.interceptors.response.use(
  (response) => {
    // 对响应数据做点什么
    return response.data;
  },
  (error) => {
    // 对响应错误做点什么
    return Promise.reject(error);
  }
);

interface Props {
  hashKey?: string;
  fileKey?: string;
  indexKey?: string;
  nameKey?: string;
  sliceHashsKey?: string;
}

interface SliceUploadOptions {
  file: File; // 源文件
  params?: Record<string, any>; // 上传额外参数
  cancelToken?: (fn: () => void) => any; // 取消上传
  sliceUploadSuccess?: () => any; // 分片传输完毕
  onBeforeMd5File?: () => any;
  onAfterMd5File?: () => any;
  sliceSize?: number; // 分片大小
  headers?: Record<string, any>;
  uploadUrl: string; // 上传地址
  mergeUrl: string; // 合并地址
  onprocess?: (uploadSize: number, totalSize: number) => any; // 进度回调
  props?: Props; // 上传参数字段
  axiosInstance?: AxiosInstance;
}

// eslint-disable-next-line @typescript-eslint/no-empty-function
const NOOP = () => {};

export const sliceUpload = async (options: SliceUploadOptions) => {
  const {
    file,
    sliceSize = 20 * 1024 * 1024,
    cancelToken,
    onBeforeMd5File = NOOP,
    onAfterMd5File = NOOP,
    uploadUrl,
    mergeUrl,
    onprocess,
    params = {},
    headers = {},
    sliceUploadSuccess,
    axiosInstance = request,
    props: {
      hashKey = 'hash',
      fileKey = 'file',
      indexKey = 'index',
      nameKey = 'name',
      // sliceHashsKey = 'sliceHashs',
    } = {},
  } = options;
  const cancelTokenQueue: any[] = [];

  function flushCancelTokenQueue() {
    cancelTokenQueue.forEach((cancel) => cancel());
    cancelTokenQueue.length = 0;
  }

  onBeforeMd5File();
  // 文件hash值，校验文件完整性
  const fileHash = (await md5(file)) as string;
  onAfterMd5File();

  // 控制权转移
  if (cancelToken) {
    cancelToken(flushCancelTokenQueue);
  }

  // 分片Blob
  const slices = [];
  for (let i = 0; i < file.size; ) {
    const blob = file.slice(i, (i += sliceSize));
    slices.push({
      [hashKey]: fileHash,
      file: blob,
    });
  }

  // uploadSize 最终值会大于文件总大小
  let uploadSize = 0;
  // 分片任务
  const tasks: Promise<any>[] = slices.map((slice, index) => {
    return new Promise((resolve1, reject1) => {
      // 组装表单数据
      const formData = new FormData();
      Object.keys(params).forEach((key) => formData.append(key, params[key]));
      formData.append(hashKey, slice[hashKey]);
      formData.append(fileKey, slice.file);
      formData.append(indexKey, `${index}`);
      // 记录单个分片已上传文件大小
      let loaded = 0;
      let token: any;
      // headers.contentType = 'multipart/form-data'
      axiosInstance
        .post(uploadUrl, formData, {
          headers,
          cancelToken: new CancelToken(function executor(c) {
            // executor 函数接收一个 cancel 函数作为参数
            token = c;
            cancelTokenQueue.push(c);
          }),
          onUploadProgress: (progressEvent) => {
            // 累加差值
            uploadSize += progressEvent.loaded - loaded;
            // 更新已上传大小
            loaded = progressEvent.loaded;
            if (onprocess) {
              onprocess(uploadSize, file.size);
            }
          },
        })
        .then(() => {
          // 上传成功删除cancel方法
          cancelTokenQueue.splice(cancelTokenQueue.indexOf(token), 1);
          resolve1(undefined);
        })
        .catch(reject1);
    });
  });

  return new Promise((resolve, reject) => {
    Promise.all(tasks)
      .then(() => {
        if (sliceUploadSuccess) {
          sliceUploadSuccess();
        }
        // 通知后台合并文件
        axiosInstance
          .post(
            mergeUrl,
            {
              ...params,
              [nameKey]: file.name,
              [hashKey]: fileHash,
            },
            {
              headers,
            }
          )
          .then((res) => {
            resolve(res);
          })
          .catch(reject);
      })
      .catch((err) => {
        // 取消未执行的任務
        flushCancelTokenQueue();
        reject(err);
      });
  });
};

export default {};
